"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var EtcdOrchestrator_1;
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const common_2 = require("@nestcloud2/common");
const etcd_1 = require("./etcd");
const inject_etcd_decorator_1 = require("./decorators/inject-etcd.decorator");
const etcd_messages_1 = require("./etcd.messages");
let EtcdOrchestrator = EtcdOrchestrator_1 = class EtcdOrchestrator {
    constructor(etcd) {
        this.etcd = etcd;
        this.keyValues = new Map();
        this.logger = new common_1.Logger(EtcdOrchestrator_1.name);
    }
    addKeyValue(target, keyValues) {
        keyValues.forEach(({ name, property, options }) => {
            const key = `${name}__${property}__${target.constructor.name}`;
            this.keyValues.set(key, { name, property, target, options });
        });
    }
    onApplicationShutdown(signal) {
        this.keyValues.forEach(item => (item.watcher ? item.watcher.cancel() : ''));
    }
    async mountKeyValues() {
        for (const item of this.keyValues.values()) {
            const { name, property, target, options = {} } = item;
            try {
                const value = await this.etcd.get(name).string();
                common_2.setValue(target, value, property, options);
            }
            catch (e) {
                this.logger.error(etcd_messages_1.ETCD_KEY_VALUE_ERROR(name), e);
            }
            const watcher = await this.etcd
                .watch()
                .key(name)
                .create();
            watcher.on('data', (res) => {
                res.events.forEach(evt => {
                    if (evt.type === 'Put') {
                        const value = evt.kv.value.toString();
                        common_2.setValue(target, value, property, options);
                    }
                    else if (evt.type === 'Delete') {
                        common_2.setValue(target, '', property, options);
                    }
                });
            });
            watcher.on('connected', async () => {
                try {
                    const value = await this.etcd.get(name).string();
                    common_2.setValue(target, value, property, options);
                }
                catch (e) {
                    this.logger.error(etcd_messages_1.ETCD_KEY_VALUE_ERROR(name), e);
                }
            });
            watcher.on('error', e => this.logger.error(etcd_messages_1.ETCD_KEY_VALUE_ERROR(name), e.message));
            item.watcher = watcher;
        }
    }
};
EtcdOrchestrator = EtcdOrchestrator_1 = __decorate([
    common_1.Injectable(),
    __param(0, inject_etcd_decorator_1.InjectEtcd()),
    __metadata("design:paramtypes", [etcd_1.Etcd])
], EtcdOrchestrator);
exports.EtcdOrchestrator = EtcdOrchestrator;
